//==============================================================
// 萔
float4x4 World;
float4x4 WView;
float4x4 WVProj;
float4 Diffuse;
float4 Ambient;
float3 Light;
texture MeshTex;
texture ToonTex;
float4 InkColor;
float InkSize;

//==============================================================
// eNX`Tv
sampler MeshSmp=sampler_state {
	Texture=<MeshTex>;
	MipFilter=LINEAR;
	MinFilter=LINEAR;
	MagFilter=LINEAR;
};
sampler ToonSmp=sampler_state {
	Texture=<ToonTex>;
	MipFilter=LINEAR;
	MinFilter=LINEAR;
	MagFilter=LINEAR;
	AddressU=CLAMP;
};

//==============================================================
// VF[_iyCgj

// _VF[_
void ToonPaintVS (
	float4 pos : POSITION,
	float3 normal : NORMAL,
	float2 mesh_uv : TEXCOORD0,
	out float4 Pos : POSITION,
	out float2 MeshUV : TEXCOORD0,
	out float2 ToonUV : TEXCOORD1
) {
	// _WAeNX`W
	Pos=mul(pos, WVProj);
	MeshUV=mesh_uv;
	
	// gD[eNX`W
	float3 w_normal=normalize(mul(normal, (float3x3)World));
	ToonUV.x=max(0, dot(w_normal, -Light))+Ambient;
	ToonUV.y=0.5;
}

// sNZVF[_
float4 ToonPaintPS(
	float2 MeshUV : TEXCOORD0,
	float2 ToonUV : TEXCOORD1
) : COLOR {
	return
		tex2D(MeshSmp, MeshUV)*
		tex2D(ToonSmp, ToonUV)*Diffuse;
}

//==============================================================
// VF[_iCNj

// _VF[_
void ToonInkVS (
	float4 pos : POSITION,
	float3 normal : NORMAL,
	out float4 Pos : POSITION
) {
	// _W
	float4 wvp_pos=mul(pos, WVProj);
	float4 wvp_normal=float4(mul(normal, (float3x3)WVProj), 0);
	Pos=wvp_pos+normalize(wvp_normal)*InkSize*wvp_pos.w;
}

// sNZVF[_
float4 ToonInkPS() : COLOR {
	return InkColor;
}

//==============================================================
// VF[_i~bNXj

// _VF[_
void ToonMixedVS (
	float4 pos : POSITION,
	float3 normal : NORMAL,
	float2 mesh_uv : TEXCOORD0,
	out float4 Pos : POSITION,
	out float2 MeshUV : TEXCOORD0,
	out float2 ToonUV : TEXCOORD1
) {
	// _WAeNX`W
	Pos=mul(pos, WVProj);
	MeshUV=mesh_uv;
	
	// gD[eNX`WiyCgj
	float3 w_normal=normalize(mul(normal, (float3x3)World));
	ToonUV.x=max(0, dot(w_normal, -Light))+Ambient;

	// gD[eNX`WiCNj
	float3 wv_pos=normalize(mul(pos, WView));
	float3 wv_normal=normalize(mul(normal, (float3x3)WView));
	ToonUV.y=dot(wv_pos, wv_normal)*0.5+0.5;
}

// sNZVF[_
float4 ToonMixedPS (
	float2 MeshUV : TEXCOORD0,
	float2 ToonUV : TEXCOORD1
) : COLOR {
	return 
		tex2D(MeshSmp, MeshUV)*
		tex2D(ToonSmp, ToonUV)*Diffuse;
}

//==============================================================
// eNjbN
technique ToonPaintInk {
	pass P0 {
		CullMode=CCW;
		VertexShader=compile vs_1_1 ToonPaintVS();
		PixelShader=compile ps_1_1 ToonPaintPS();
	}
	pass P1 {
		CullMode=CW;
		VertexShader=compile vs_1_1 ToonInkVS();
		PixelShader=compile ps_1_1 ToonInkPS();
	}
}
technique ToonMixed {
	pass P0 {
		VertexShader=compile vs_1_1 ToonMixedVS();
		PixelShader=compile ps_1_1 ToonMixedPS();
	}
}
