#include "Texture.h"
#include <d3dx9.h>


//==============================================================
// _`

// \
struct VERTEX
{
	// ϊςݒ_
	FLOAT X, Y, Z, RHW;

	// fBt[YF
	D3DCOLOR Diffuse;

	// eNX`W
	FLOAT U, V;
};

// FVF
#define D3DFVF_VERTEX (D3DFVF_XYZRHW|D3DFVF_DIFFUSE|D3DFVF_TEX1)


//==============================================================
// eNX`

// RXgN^
CTexture::CTexture(CTextureGroup* group, int w, int h)
:	Group(group), W(w), H(h), Texture(NULL)
{
	// o^
	Group->Texture.push_back(this);

	// eNX`̍쐬
	Group->Device->CreateTexture(
		w, h, 1, D3DUSAGE_RENDERTARGET, 
		D3DFMT_A8R8G8B8, D3DPOOL_DEFAULT, &Texture, NULL);
}

// fXgN^
CTexture::~CTexture() {
	if (Texture) delete Texture;
}

// `
void CTexture::Draw(
	float x0, float y0, float x1, float y1, 
	float u0, float v0, float u1, float v1, 
	D3DCOLOR diffuse
) {
	LPDIRECT3DDEVICE9 device=Group->Device;

	// eNX`@̐ݒ
	device->SetTextureStageState(0, D3DTSS_COLOROP, D3DTOP_MODULATE);
	device->SetTextureStageState(0, D3DTSS_COLORARG1, D3DTA_TEXTURE);
	device->SetTextureStageState(0, D3DTSS_COLORARG2, D3DTA_DIFFUSE);
	device->SetTextureStageState(0, D3DTSS_ALPHAOP, D3DTOP_MODULATE);
	device->SetTextureStageState(0, D3DTSS_ALPHAARG1, D3DTA_TEXTURE);
	device->SetTextureStageState(0, D3DTSS_ALPHAARG2, D3DTA_DIFFUSE);

	// _obt@foCXɐݒ
	LPDIRECT3DVERTEXBUFFER9 vb=Group->VertexBuffer;
	device->SetStreamSource(0, vb, 0, sizeof(VERTEX));
	device->SetFVF(D3DFVF_VERTEX);

	// _obt@̃bN
	VERTEX* v;
	if (SUCCEEDED(
		vb->Lock(0, 4*sizeof(VERTEX), (void**)&v, 0)
	)) {
		// _W̐ݒ
		v[0].X=x0; v[0].Y=y0;
		v[1].X=x1; v[1].Y=y0;
		v[2].X=x0; v[2].Y=y1;
		v[3].X=x1; v[3].Y=y1;

		// eNX`W̐ݒ
		v[0].U=u0; v[0].V=v0;
		v[1].U=u1; v[1].V=v0;
		v[2].U=u0; v[2].V=v1;
		v[3].U=u1; v[3].V=v1;

		// Z, RHW, _F̐ݒ
		for (int i=0; i<4; i++) {
			v[i].Z=0;
			v[i].RHW=1; 
			v[i].Diffuse=diffuse;
		}

		// _obt@̉
		vb->Unlock();

		// eNX`̑Iƕ`
		device->SetTexture(0, Texture);
		device->DrawPrimitive(D3DPT_TRIANGLESTRIP, 0, 2);
	}
}

// eNX`hԂp̊֐
static VOID WINAPI FillTexture(
	D3DXVECTOR4* pOut, const D3DXVECTOR2* pTexCoord, 
	const D3DXVECTOR2* pTexelSize, LPVOID pData
) {
	*pOut=*(D3DXVECTOR4*)pData;
}

// hԂ
void CTexture::Fill(D3DXVECTOR4* color) {
	D3DXFillTexture(Texture, FillTexture, &color);
}


//==============================================================
// eNX`O[v
CTextureGroup::CTextureGroup(
	LPDIRECT3DDEVICE9 device, int max_w, int max_h
) :	Device(device), MaxW(max_w), MaxH(max_h), 
	VertexBuffer(NULL), DepthBuffer(NULL)
{
	// _obt@̍쐬
	device->CreateVertexBuffer(
		4*sizeof(VERTEX), 0, D3DFVF_VERTEX, 
		D3DPOOL_MANAGED, &VertexBuffer, NULL);

	// [xobt@̍쐬
	device->CreateDepthStencilSurface(
		max_w, max_h, D3DFMT_D16, D3DMULTISAMPLE_NONE, 
		0, TRUE, &DepthBuffer, NULL);
}

// fXgN^
CTextureGroup::~CTextureGroup() {
	for (int i=0, n=Texture.size(); i<n; i++) delete Texture[i];
	if (VertexBuffer) VertexBuffer->Release();
	if (DepthBuffer) DepthBuffer->Release();
}
